use strict;
use warnings;

package directoryusage; 
# 
# directoryusage -- Produces a list of directories and the space used#                   for the last n days.  Generates a comma separated
#                   by each directory.  Output to STDOUT is a comma-separated file that
#                   file with path, size and modification date.
#
#
# Usage:
#
#         directoryusage PATH > STDOUT
#         
#         PATH = Directory path to be scanned
#                Only works on *directories*;  if a file is scanned
#                the program will terminate
#
# Author:
#
#         David Moisan
#         Salem Access TV
#         7 July 2001
#
#
#
# Defaults:
#
# None

# Globals:

$directoryusage::Dircount = 0;  # Count of directories found
$directoryusage::Filecount = 0; # Count of files found

# Usage check
# At least scandir must be specified

unless (defined $ARGV[0]) { 
    die ("Usage: directoryusage PATH");
    }      

# If date not specified, use default

my $path = $ARGV[0];

# Initialize;  print header

open STDOUT, ">";

my $localtimestring = localtime(time());
my $size = 0; #Size of all directories and files

print STDOUT "Scan of $path on $localtimestring \n";
print STDOUT "Path, Size, Modification \n";

# Scan directory recursively
$size = fileusage($path);

# Finish up:  Write total stats and close file

print STDOUT ($directoryusage::Filecount," files in ",
              $directoryusage::Dircount," directories \n");
print STDOUT ("Total size of $path is $size bytes \n");
close STDOUT;
exit;


# Subroutine:  fileusage($path)
#
# Prints directory information for any file modified  
# after the specified date.
#
# Parameters:   $path   File or directory path to check
#               $date   File creation/modification/access date to check
#

sub fileusage {

    my $path = $_[0];
    my $size = 0;

    my @contents;  # Directory list

    # Get a fully qualified file name (with path)
 
    if (-f $path) {
        #print STDERR ("$path is a file--processing! \n");
        $size = getfilesize($path);
        $directoryusage::Filecount++;
        return $size;
        }
       
       # If it's a directory, keep recursing down 

    elsif (-d $path) {
        #print STDERR ("$path is a Directory--recursing\n");
        opendir(X, $path);
        @contents = readdir(X);
        closedir(X);
        $directoryusage::Dircount++;
        foreach my $file (@contents) {
           # Skip '.' and '..' directories
            next if ($file eq "." || $file eq "..");
            $size += &fileusage(fullfilepath($path, $file));
            }
            printdirinfo($path, $size);
            return $size;
        }

    else {
        print STDERR ("$path Not Found \n");
        }
    return;
}

# Subroutine:  getfilesize($path)
#
# Retrieves file size of given path. 
#
# 

sub getfilesize {
    my $path = $_[0];
    my @filestat = stat($path);

    return $filestat[7];  # return size
    }

#
# Subroutine:  printdirinfo 
# Print path, size (of files within) and modification date of a directory
#

sub printdirinfo($path,$size) {

    my $path = $_[0];
    my $size = $_[1];
    my @filestats = stat($path);

    my $modifydate = localtime($filestats[9]);


    print STDOUT ($path, ",",
                  $size, ",", 
                  $modifydate,",",
                  "\n");
    return;
    }


sub fullfilepath {

# Subroutine fullfilepath ($path, $file)
# Returns complete file path when a directory and file are 
# specified (assuming the file is in that same directory!)
#
# FOR WIN32:  Uses backslash file separator
#
#
#   $path   Directory path
#   $name   File name
#
# Returns scalar:
# 
#   $fullpath   Fully qualified directory path
#
#
    my $FILESEPARATOR = "\\";  # Win32 FAT/NTFS file separator

    my $path = $_[0];
    my $name = $_[1];
    my $fullpath;

    #debug
    #print STDERR ('$path ',$path, "\n");
    #print STDERR ('$name ',$name, "\n");
    unless ($name) {
        return $fullpath; 
        }
    
    # If there's a trailing file separator,
    # omit it

    elsif (substr ($path,-1) eq $FILESEPARATOR) {
            $fullpath = $path.$name;
            return $fullpath;
            }
        else {  # Include the file separator
            $fullpath = $path.$FILESEPARATOR.$name;
            return $fullpath;
            }

    return;
    }


