use strict;
use warnings;

package latestfiles; 
# 
# latestfiles.pl -- Produces a list of most-recently used files
#                   for the last n days.  Generates a comma separated
#                   file with file number, name, path, size and content date
#                   Sends output to STDOUT.
#                   
#
#
# Usage:
#
#         latestfiles PATH DATE  > STDOUT
#         
#         PATH = Directory path to be scanned
#         DAYS = last n days.  Optional:  if not specified,
#         default is 1 day.
#
# 
#
#
# Author:
#
#         David Moisan
#         Salem Access TV
#         7 July 2001
#
#
#
# Defaults:

 my $DEFAULTDAYS = 1;  # default number of days back to compare


# Usage check
# At least scandir must be specified

unless (defined $ARGV[0]) { 
    die ("Usage: latestfiles SCANDIR [DAYS]");
    }      

# If date not specified, use default

my $scandir = $ARGV[0];

my $days = $ARGV[1];

unless ($days) {
        $days = $DEFAULTDAYS;
        }

# Initialize;  reset file number and print header


$latestfiles::Filenum=0; # Global
$latestfiles::Filedatetype = 2; # File compare type = Modification Date
$latestfiles::totalsize=0;

my $pastdate = daysago(time,$days);

open STDOUT, ">";

my $localtimestring = localtime(time());
my $pasttimestring = localtime($pastdate);

print STDOUT ("Scan of $scandir on $localtimestring since $pasttimestring");
print STDOUT ("\n");
print STDOUT "Number, Path, Size, Creation, Access, Modification \n";

# Scan directory recursively

filedatescan($scandir,$pastdate);

# Finish up:  Write total stats and close file

print STDOUT ("Total number of files in ", $scandir,": ,",$latestfiles::Filenum,"\n");
print STDOUT ("Total size of changed files, ",$latestfiles::totalsize,"\n");

close STDOUT;
exit;


# Subroutine:  filedatescan($path,$days)
#
# Prints directory information for any file modified  
# after the specified date.
#
# Parameters:   $path   File or directory path to check
#               $date   File creation/modification/access date to check
#
# Global Parameter: 
#               $latestfiles::Filedatetype  
#               Type of file date to check for
#

sub filedatescan {

    my $path = $_[0];
    my $days = $_[1];

    my @contents;  # Directory list

    # Get a fully qualified file name (with path)
 
    if (-f $path) {
        #print STDOUT ("$path is a file--processing! \n");
        if (checkfiledate($path,$days,$latestfiles::Filedatetype)) {
            printfilerecord($path);
            }
        }
       
       # If it's a directory, keep recursing down 

    elsif (-d $path) {
        #print STDOUT ("$path is a Directory--recursing\n");
        opendir(X, $path);
        @contents = readdir(X);
        closedir(X);
        foreach my $file (@contents) {
           # Skip '.' and '..' directories
            next if ($file eq "." || $file eq "..");
            filedatescan(fullfilepath($path, $file), $days);
            }
        return;
        }

    else {
        print STDERR ("$path Not Found \n");
        }
    return;
}

#
# Subroutine:  checkfiledate ($path,$date,$filedatetype)
#
# Checks file creation, access or modification date and 
# returns -1 (true) if file is same or newer than the 
# specified epoch date
#
#   $path           Full file spec of file to check
#   $date           Epoch date to compare against
#   $filedatetype   Type of file date comparison to make:
#                   1   CREATIONTIME    Creation time
#                   2   MODIFYTIME      Modification time
#                   3   ACCESSTIME      Access time
#
#

sub checkfiledate {

# Constants

    my $CREATIONTIME = 1;
    my $MODIFYTIME = 2;
    my $ACCESSTIME = 3;

    # get ($path, $date, $filedatetype) 

    my $path = $_[0];
    my $date = $_[1];
    my $filedatetype = $_[2];

    # local vars

    my @filestats = stat($path);

    my $modifydate = $filestats[9];
    my $createdate = $filestats[10];
    my $accessdate = $filestats[8];

    #debug
    #print STDOUT ("File: $path \n",
    #   "Created: ",localtime($createdate), "\n",
    #   "Accessed: ",localtime($accessdate), "\n",
    #   "Modified: ",localtime($modifydate), "\n\n");

    my $comparedate;

    $comparedate=$modifydate; # Default comparison is to modify date

    if ($filedatetype==$CREATIONTIME) {$comparedate=$createdate;}
    if ($filedatetype==$MODIFYTIME) {$comparedate=$modifydate;}
    if ($filedatetype==$ACCESSTIME) {$comparedate=$accessdate;}


    if ($comparedate >= $date) {return 1;} 
        else {return 0;}
    }

sub printfilerecord {

# subroutine printfilerecord($path)
#
# Prints a comma-separated record of the current file ($path)
# Data includes: 
#
#   Number, Path, Size, Creation, Access, Modification
#     
#
#
    my $path = $_[0];

    my @filestats = stat($path);


    my $modifydate = localtime($filestats[9]);
    my $createdate = localtime($filestats[10]);
    my $accessdate = localtime($filestats[8]);
    my $filesize = $filestats[7];


    print STDOUT ($latestfiles::Filenum++,",", 
            $path, ",",
            $filesize, ",", 
            $createdate,",",
            $accessdate,",",
            $modifydate,",",
            "\n");
            $latestfiles::totalsize += $filesize;
    return;
    }


sub fullfilepath {

# Subroutine fullfilepath ($path, $file)
# Returns complete file path when a directory and file are 
# specified (assuming the file is in that same directory!)
#
# FOR WIN32:  Uses backslash file separator
#
#
#   $path   Directory path
#   $name   File name
#
# Returns scalar:
# 
#   $fullpath   Fully qualified directory path
#
#
    my $FILESEPARATOR = "\\";  # Win32 FAT/NTFS file separator

    my $path = $_[0];
    my $name = $_[1];
    my $fullpath;

    #debug
    #print STDOUT ('$path ',$path, "\n");
    #print STDOUT ('$name ',$name, "\n");
    unless ($name) {
        return $fullpath; 
        }
    
    # If there's a trailing file separator,
    # omit it

    elsif (substr ($path,-1) eq $FILESEPARATOR) {
            $fullpath = $path.$name;
            return $fullpath;
            }
        else {  # Include the file separator
            $fullpath = $path.$FILESEPARATOR.$name;
            return $fullpath;
            }

    return;
    }

#
#
#
#
#
#
# 
# sub daysago($time, $days)
#
# Returns the epoch time from an arbitrary number of days ago
# or yesterday, if no number of days specified
# 
#   $time   Epoch time. Defaults to current time if no days 
#           specified
#
#   $days   Number of days to go back;  if not specified, 1 day
#           ("yesterday") is default.
#
# Modified from Russ Allbery's yesterday script by David Moisan
# 8-July-2001
#
#

sub daysago {
    my $now  = defined $_[0] ? $_[0] : time;
    my $days = defined $_[1] ? $_[1] : 1;
    my $then = $now - (60 * 60 * 24)* $days;
    my $ndst = (localtime $now)[8] > 0;
    my $tdst = (localtime $then)[8] > 0;
    return $then - ($tdst - $ndst) * 60 * 60;
    }

    # Should give you "this time n days ago" in seconds since epoch relative to
    # the first argument or the current time if no argument is given and
    # suitable for passing to localtime or whatever else you need to do with
    # it.  $ndst is whether we're currently in daylight savings time; $tdst is
    # whether the point 24 hours ago was in daylight savings time.  If $tdst
    # and $ndst are the same, a boundary wasn't crossed, and the correction
    # will subtract 0.  If $tdst is 1 and $ndst is 0, subtract an hour more
    # from yesterday's time since we gained an extra hour while going off
    # daylight savings time.  If $tdst is 0 and $ndst is 1, subtract a
    # negative hour (add an hour) to yesterday's time since we lost an hour.
    #
    # All of this is because during those days when one switches off or onto
    # DST, a "day" isn't 24 hours long; it's either 23 or 25.
    #
    # The explicit settings of $ndst and $tdst are necessary because localtime
    # only says it returns the system tm struct, and the system tm struct at
    # least on Solaris doesn't guarantee any particular positive value (like,
    # say, 1) for isdst, just a positive value.  And that value can
    # potentially be negative, if DST information isn't available (this sub
    # just treats those cases like no DST).
    #
    # Note that between 2am and 3am on the day after the time zone switches
    # off daylight savings time, the exact hour of "yesterday" corresponding
    # to the current hour is not clearly defined.  Note also that if used
    # between 2am and 3am the day after the change to daylight savings time,
    # the result will be between 3am and 4am of the previous day; it's
    # arguable whether this is correct.
    #
    # This sub does not attempt to deal with leap seconds (most things don't).
    #
    # Copyright relinquished 1999 by Russ Allbery <rra@stanford.edu>
    # This code is in the public domain

